/*
 * Pidgin Toolbar Icons
 * Copyright (C) 2010 Konrad Gräfe
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301, USA.
 */

#include "toolbar_shrink.h"

#include "iconification.h"

#include <gtk/gtk.h>
#include <gtkconv.h>
#include <debug.h>
#include <gtkimhtmltoolbar.h>

extern PurplePlugin *plugin;

static gchar **button_text;
static gchar **prefs;

static void force_hidden(GtkWidget *widget, GdkEventVisibility *event, gpointer null) {
	gtk_widget_hide(widget);
}


static void widget_set_iconification(GtkWidget *widget, gpointer hide) {
	gulong *handler_id;
	int button;
	/* TODO: rename variables... */
	gboolean really_hide;

	if(GTK_IS_LABEL(widget)) {
		handler_id = g_object_get_data(G_OBJECT(widget), DATA_KEY_HANDLER);

		if(GPOINTER_TO_BOOLEAN(hide)) {
			really_hide = FALSE;

			for(button = 0; button < sizeof(ToolbarButtons) && !really_hide; button++) {
				really_hide = (purple_prefs_get_bool(prefs[button]) && purple_strequal(gtk_label_get_label(GTK_LABEL(widget)), button_text[button]));
			}

			if(really_hide) {
				gtk_widget_hide(widget);

				if(!handler_id) {
					handler_id = g_malloc(sizeof(gulong));
					*handler_id = g_signal_connect(GTK_WIDGET(widget), "show", G_CALLBACK(force_hidden), hide);
					g_object_set_data(G_OBJECT(widget), DATA_KEY_HANDLER, handler_id);
				}
			}
		} else {
			if(handler_id) {
				g_signal_handler_disconnect(G_OBJECT(widget), *handler_id);
				g_object_set_data(G_OBJECT(widget), DATA_KEY_HANDLER, NULL);
				g_free(handler_id);
			}

			gtk_widget_show(widget);
		}
	} else if(GTK_IS_CONTAINER(widget)) {
		gtk_container_foreach(GTK_CONTAINER(widget), widget_set_iconification, hide);
	}
}


static void toolbar_set_iconification(PurpleConversation *conv, gpointer hide) {
	PidginConversation *gtkconv;
	GtkWidget *toolbar;
	GtkWidget *lean_view_box;

	if(!PIDGIN_IS_PIDGIN_CONVERSATION(conv)) return;

	gtkconv = PIDGIN_CONVERSATION(conv);

	toolbar = gtkconv->toolbar;
	if(!toolbar) return;
	
	lean_view_box = g_object_get_data(G_OBJECT(toolbar), "lean-view");
	if(!lean_view_box) return;

	gtk_container_foreach(GTK_CONTAINER(lean_view_box), widget_set_iconification, hide);
}

void iconification_init(void) {
	GList *l;

	button_text = g_malloc(sizeof(gchar *) * sizeof(ToolbarButtons));
	prefs = g_malloc(sizeof(gchar *) * sizeof(ToolbarButtons));

	/* get Pidgins translations */
	button_text[TOOLBAR_BUTTON_FONT] = dgettext("pidgin", "_Font");
	button_text[TOOLBAR_BUTTON_INSERT] = dgettext("pidgin", "_Insert");
	button_text[TOOLBAR_BUTTON_SMILE] = dgettext("pidgin", "_Smile!");
	button_text[TOOLBAR_BUTTON_ATTENTION] = dgettext("pidgin", "_Attention!");

	prefs[TOOLBAR_BUTTON_FONT] = PLUGIN_PREFS_PREFIX "/shrink_font_button";
	prefs[TOOLBAR_BUTTON_INSERT] = PLUGIN_PREFS_PREFIX "/shrink_insert_button";
	prefs[TOOLBAR_BUTTON_SMILE] = PLUGIN_PREFS_PREFIX "/shrink_smile_button";
	prefs[TOOLBAR_BUTTON_ATTENTION] = PLUGIN_PREFS_PREFIX "/shrink_attention_button";

	purple_signal_connect(purple_conversations_get_handle(), "conversation-created", plugin, PURPLE_CALLBACK(toolbar_set_iconification), GBOOLEAN_TO_POINTER(TRUE));

	for(l = purple_get_conversations(); l != NULL; l = l->next) {
		toolbar_set_iconification(l->data, GBOOLEAN_TO_POINTER(TRUE));
	}
}

void iconification_uninit(void) {
	GList *l;

	g_free(button_text);
	g_free(prefs);

	for(l = purple_get_conversations(); l != NULL; l = l->next) {
		toolbar_set_iconification(l->data, GBOOLEAN_TO_POINTER(FALSE));
	}
}

